@@ -6,8 +6,8 @@ from PyQt4.QtCore import *
 import PyQt4.QtCore as QtCore
 
 import electrum
-from electrum.wallet import Wallet
-from electrum.util import UserCancelled
+from electrum import Wallet, WalletStorage
+from electrum.util import UserCancelled, InvalidPassword
 from electrum.base_wizard import BaseWizard
 from electrum.i18n import _
 
@@ -96,14 +96,11 @@ def wizard_dialog(func):
 class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
 
     def __init__(self, config, app, plugins, storage):
-
         BaseWizard.__init__(self, config, storage)
         QDialog.__init__(self, None)
-
         self.setWindowTitle('Electrum  -  ' + _('Install Wizard'))
         self.app = app
         self.config = config
-
         # Set for base base class
         self.plugins = plugins
         self.language_for_seed = config.get('language')
@@ -112,6 +109,7 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
         self.title = QLabel()
         self.main_widget = QWidget()
         self.back_button = QPushButton(_("Back"), self)
+        self.back_button.setText(_('Back') if self.can_go_back() else _('Cancel'))
         self.next_button = QPushButton(_("Next"), self)
         self.next_button.setDefault(True)
         self.logo = QLabel()
@@ -147,6 +145,90 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
 
     def run_and_get_wallet(self):
 
+        vbox = QVBoxLayout()
+        hbox = QHBoxLayout()
+        hbox.addWidget(QLabel(_('Wallet') + ':'))
+        self.name_e = QLineEdit()
+        hbox.addWidget(self.name_e)
+        button = QPushButton(_('Choose...'))
+        hbox.addWidget(button)
+        vbox.addLayout(hbox)
+
+        self.msg_label = QLabel('')
+        vbox.addWidget(self.msg_label)
+        hbox2 = QHBoxLayout()
+        self.pw_e = QLineEdit('', self)
+        self.pw_e.setFixedWidth(150)
+        self.pw_e.setEchoMode(2)
+        self.pw_label = QLabel(_('Password') + ':')
+        hbox2.addWidget(self.pw_label)
+        hbox2.addWidget(self.pw_e)
+        hbox2.addStretch()
+        vbox.addLayout(hbox2)
+        self.set_layout(vbox, title=_('Electrum wallet'))
+
+        wallet_folder = os.path.dirname(self.storage.path)
+
+        def on_choose():
+            path = unicode(QFileDialog.getOpenFileName(self, "Select your wallet file", wallet_folder))
+            if path:
+                self.name_e.setText(path)
+
+        def on_filename(filename):
+            filename = unicode(filename)
+            path = os.path.join(wallet_folder, filename.encode('utf8'))
+            try:
+                self.storage = WalletStorage(path)
+            except IOError:
+                self.storage = None
+            if self.storage:
+                if not self.storage.file_exists():
+                    msg =_("This file does not exist.") + '\n' \
+                          + _("Press 'Next' to create this wallet, or choose another file.")
+                    pw = False
+                elif self.storage.file_exists() and self.storage.is_encrypted():
+                    msg = _("This file is encrypted.") + '\n' + _('Enter your password or choose another file.')
+                    pw = True
+                else:
+                    msg = _("Press 'Next' to open this wallet.")
+                    pw = False
+            else:
+                msg = _('Cannot read file')
+                pw = False
+            self.msg_label.setText(msg)
+            if pw:
+                self.pw_label.show()
+                self.pw_e.show()
+                self.pw_e.setFocus()
+            else:
+                self.pw_label.hide()
+                self.pw_e.hide()
+
+        button.clicked.connect(on_choose)
+        self.name_e.textChanged.connect(on_filename)
+        n = os.path.basename(self.storage.path)
+        self.name_e.setText(n.decode('utf8'))
+
+        while True:
+            if self.storage.file_exists() and not self.storage.is_encrypted():
+                break
+            if self.loop.exec_() != 2:  # 2 = next
+                return
+            if not self.storage.file_exists():
+                break
+            if self.storage.file_exists() and self.storage.is_encrypted():
+                password = unicode(self.pw_e.text())
+                try:
+                    self.storage.decrypt(password)
+                    break
+                except InvalidPassword as e:
+                    QMessageBox.information(None, _('Error'), str(e), _('OK'))
+                    continue
+                except BaseException as e:
+                    traceback.print_exc(file=sys.stdout)
+                    QMessageBox.information(None, _('Error'), str(e), _('OK'))
+                    return
+
         path = self.storage.path
         if self.storage.requires_split():
             self.hide()
@@ -169,7 +251,6 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
             self.storage.upgrade()
             self.show_warning(_('Your wallet was upgraded successfully'))
             self.wallet = Wallet(self.storage)
-            self.terminate()
             return self.wallet
 
         action = self.storage.get_action()
@@ -188,6 +269,10 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
             self.run(action)
             return self.wallet
 
+        self.wallet = Wallet(self.storage)
+        return self.wallet
+
+
 
     def finished(self):
         """Called in hardware client wrapper, in order to close popups."""
@@ -203,8 +288,7 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
         self.logo.setPixmap(QPixmap(filename).scaledToWidth(60))
         return prior_filename
 
-    def set_main_layout(self, layout, title=None, raise_on_cancel=True,
-                        next_enabled=True):
+    def set_layout(self, layout, title=None, next_enabled=True):
         self.title.setText("<b>%s</b>"%title if title else "")
         self.title.setVisible(bool(title))
         # Get rid of any prior layout by assigning it to a temporary widget
@@ -218,6 +302,10 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
             self.next_button.setFocus()
         self.main_widget.setVisible(True)
         self.please_wait.setVisible(False)
+
+    def exec_layout(self, layout, title=None, raise_on_cancel=True,
+                        next_enabled=True):
+        self.set_layout(layout, title, next_enabled)
         result = self.loop.exec_()
         if not result and raise_on_cancel:
             raise UserCancelled
@@ -241,12 +329,12 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
 
     def text_input(self, title, message, is_valid):
         slayout = KeysLayout(parent=self, title=message, is_valid=is_valid)
-        self.set_main_layout(slayout, title, next_enabled=False)
+        self.exec_layout(slayout, title, next_enabled=False)
         return slayout.get_text()
 
     def seed_input(self, title, message, is_seed, options):
         slayout = SeedLayout(title=message, is_seed=is_seed, options=options, parent=self)
-        self.set_main_layout(slayout, title, next_enabled=False)
+        self.exec_layout(slayout, title, next_enabled=False)
         return slayout.get_seed(), slayout.is_bip39, slayout.is_ext
 
     @wizard_dialog
@@ -264,9 +352,14 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
 
     @wizard_dialog
     def restore_seed_dialog(self, run_next, test):
+        options = []
+        if self.opt_ext:
+            options.append('ext')
+        if self.opt_bip39:
+            options.append('bip39')
         title = _('Enter Seed')
         message = _('Please enter your seed phrase in order to restore your wallet.')
-        return self.seed_input(title, message, test, ['ext', 'bip39'])
+        return self.seed_input(title, message, test, options)
 
     @wizard_dialog
     def confirm_seed_dialog(self, run_next, test):
@@ -284,13 +377,14 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
     def show_seed_dialog(self, run_next, seed_text):
         title =  _("Your wallet generation seed is:")
         slayout = SeedLayout(seed=seed_text, title=title, msg=True, options=['ext'])
-        self.set_main_layout(slayout)
+        self.exec_layout(slayout)
         return slayout.is_ext
 
     def pw_layout(self, msg, kind):
         playout = PasswordLayout(None, msg, kind, self.next_button)
-        self.set_main_layout(playout.layout())
-        return playout.new_password()
+        playout.encrypt_cb.setChecked(True)
+        self.exec_layout(playout.layout())
+        return playout.new_password(), playout.encrypt_cb.isChecked()
 
     @wizard_dialog
     def request_password(self, run_next):
@@ -326,7 +420,7 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
     def confirm(self, message, title):
         vbox = QVBoxLayout()
         vbox.addWidget(WWLabel(message))
-        self.set_main_layout(vbox, title)
+        self.exec_layout(vbox, title)
 
     @wizard_dialog
     def action_dialog(self, action, run_next):
@@ -340,6 +434,7 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
         self.refresh_gui()
         t = threading.Thread(target = task)
         t.start()
+        t.join()
 
     @wizard_dialog
     def choice_dialog(self, title, message, choices, run_next):
@@ -348,7 +443,7 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
         clayout = ChoicesLayout(message, c_titles)
         vbox = QVBoxLayout()
         vbox.addLayout(clayout.layout())
-        self.set_main_layout(vbox, title)
+        self.exec_layout(vbox, title)
         action = c_values[clayout.selected_index()]
         return action
 
@@ -357,7 +452,7 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
         clayout = ChoicesLayout(msg, choices)
         vbox = QVBoxLayout()
         vbox.addLayout(clayout.layout())
-        self.set_main_layout(vbox, '')
+        self.exec_layout(vbox, '')
         return clayout.selected_index()
 
     @wizard_dialog
@@ -367,11 +462,11 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
         line = QLineEdit()
         line.setText(default)
         def f(text):
-            self.next_button.setEnabled(test(text))
+            self.next_button.setEnabled(test(unicode(text)))
         line.textEdited.connect(f)
         vbox.addWidget(line)
         vbox.addWidget(WWLabel(warning))
-        self.set_main_layout(vbox, title, next_enabled=test(default))
+        self.exec_layout(vbox, title, next_enabled=test(default))
         return ' '.join(unicode(line.text()).split())
 
     @wizard_dialog
@@ -383,7 +478,7 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
         vbox = QVBoxLayout()
         layout = SeedLayout(xpub, title=msg, icon=False)
         vbox.addLayout(layout.layout())
-        self.set_main_layout(vbox, _('Master Public Key'))
+        self.exec_layout(vbox, _('Master Public Key'))
         return None
 
     def init_network(self, network):
@@ -396,19 +491,16 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
         choices = [_("Auto connect"), _("Select server manually")]
         title = _("How do you want to connect to a server? ")
         clayout = ChoicesLayout(message, choices)
-        self.set_main_layout(clayout.layout(), title)
+        self.back_button.setText(_('Cancel'))
+        self.exec_layout(clayout.layout(), title)
         r = clayout.selected_index()
-        if r == 0:
-            auto_connect = True
-        elif r == 1:
-            auto_connect = True
+        if r == 1:
             nlayout = NetworkChoiceLayout(network, self.config, wizard=True)
-            if self.set_main_layout(nlayout.layout()):
-                auto_connect = False
+            if self.exec_layout(nlayout.layout()):
+                nlayout.accept()
         else:
-            auto_connect = True
-        network.auto_connect = auto_connect
-        self.config.set_key('auto_connect', auto_connect, True)
+            network.auto_connect = True
+            self.config.set_key('auto_connect', True, True)
 
     @wizard_dialog
     def multisig_dialog(self, run_next):
@@ -443,7 +535,7 @@ class InstallWizard(QDialog, MessageBoxMixin, BaseWizard):
         vbox.addWidget(cw)
         vbox.addWidget(WWLabel(_("Choose the number of signatures needed to unlock funds in your wallet:")))
         vbox.addLayout(grid)
-        self.set_main_layout(vbox, _("Multi-Signature Wallet"))
+        self.exec_layout(vbox, _("Multi-Signature Wallet"))
         m = int(m_edit.value())
         n = int(n_edit.value())
         return (m, n)
