@@ -26,6 +26,7 @@
 import os
 import bitcoin
 import keystore
+from keystore import bip44_derivation
 from wallet import Wallet, Imported_Wallet, Standard_Wallet, Multisig_Wallet, wallet_types
 from i18n import _
 from plugins import run_hook
@@ -72,7 +73,7 @@ class BaseWizard(object):
 
     def new(self):
         name = os.path.basename(self.storage.path)
-        title = _("Create '%s'"%name)
+        title = _("Create") + ' ' + name.decode('utf8')
         message = '\n'.join([
             _("What kind of wallet do you want to create?")
         ])
@@ -189,9 +190,11 @@ class BaseWizard(object):
                 continue
             devices += map(lambda x: (name, x), u)
         if not devices:
-            msg = '\n'.join([
-                _('No hardware device detected.'),
-                _('To trigger a rescan, press \'next\'.'),
+            msg = ''.join([
+                _('No hardware device detected.') + '\n',
+                _('To trigger a rescan, press \'Next\'.') + '\n\n',
+                _('If your device is not detected on Windows, go to "Settings", "Devices", "Connected devices", and do "Remove device". Then, plug your device again.') + ' ',
+                _('On Linux, you might have to add a new permission to your udev rules.'),
             ])
             self.confirm_dialog(title=title, message=msg, run_next= lambda x: self.choose_hw_device())
             return
@@ -219,22 +222,16 @@ class BaseWizard(object):
             # This is partially compatible with BIP45; assumes index=0
             self.on_hw_derivation(name, device_info, "m/45'/0")
         else:
-            from keystore import bip44_derivation
-            f = lambda x: self.run('on_hw_derivation', name, device_info, bip44_derivation(int(x)))
-            self.account_id_dialog(f)
+            f = lambda x: self.run('on_hw_derivation', name, device_info, str(x))
+            self.derivation_dialog(f)
 
-    def account_id_dialog(self, f):
+    def derivation_dialog(self, f):
+        default = bip44_derivation(0)
         message = '\n'.join([
-            _('Enter your BIP44 account number here.'),
-            _('If you are not sure what this is, leave this field to zero.')
+            _('Enter your wallet derivation here.'),
+            _('If you are not sure what this is, leave this field unchanged.')
         ])
-        def is_int(x):
-            try:
-                int(x)
-                return True
-            except:
-                return False
-        self.line_dialog(run_next=f, title=_('Account Number'), message=message, default='0', test=is_int)
+        self.line_dialog(run_next=f, title=_('Derivation'), message=message, default=default, test=bitcoin.is_bip32_derivation)
 
     def on_hw_derivation(self, name, device_info, derivation):
         from keystore import hardware_keystore
@@ -291,17 +288,16 @@ class BaseWizard(object):
             raise BaseException('Unknown seed type', seed_type)
 
     def on_restore_bip39(self, seed, passphrase):
-        f = lambda x: self.run('on_bip44', seed, passphrase, int(x))
-        self.account_id_dialog(f)
+        f = lambda x: self.run('on_bip44', seed, passphrase, str(x))
+        self.derivation_dialog(f)
 
     def create_keystore(self, seed, passphrase):
         k = keystore.from_seed(seed, passphrase)
         self.on_keystore(k)
 
-    def on_bip44(self, seed, passphrase, account_id):
+    def on_bip44(self, seed, passphrase, derivation):
         k = keystore.BIP32_KeyStore({})
         bip32_seed = keystore.bip39_to_seed(seed, passphrase)
-        derivation = "m/44'/0'/%d'"%account_id
         k.add_xprv_from_seed(bip32_seed, 0, derivation)
         self.on_keystore(k)
 
